@@ -575,6 +575,11 @@ identifyParticles <- function (sbg,threshold=-0.1,pixelRange=NULL,
        stop(c("\n \t No particle detected in any of the frames. ",
               "Try different thresholds? \n"))
     }
+    if (any(totalPart == 0)) {
+      warning(c("\n \t No particle detected in at least one of the frames.
+             You will run into problems with particle tracking.
+             Try different thresholds? \n"))
+    }
 
     ##method to calculate shape index or aggregation indexes
   	#a = area of the patch in number of cells
@@ -600,27 +605,29 @@ identifyParticles <- function (sbg,threshold=-0.1,pixelRange=NULL,
                                  'frac.dim.index','core.area.index',
                                   paste0('mu',c('R','G','B')),'x','y','frame')
     for (i in n) {
-       if (i == 1) loc <- 1:totalPart[i]
-       if (i > 1) {
-         cum <- sum(totalPart[1:(i-1)])+1
-         loc <- cum:(cum+totalPart[i]-1)
-       }
+      if (totalPart[i] > 0) {
+        if (i == 1) loc <- 1:totalPart[i]
+        if (i > 1) {
+          cum <- sum(totalPart[1:(i-1)])+1
+          loc <- cum:(cum+totalPart[i]-1)
+        }
 
-       IDs <- sort(as.numeric(stats::na.omit(unique(c(A[,,i])))))[-1]
-       particleStats[loc,1:5] <- .Call('projectedPS',A[,,i],IDs,
-                                       PACKAGE='trackdem')
+        IDs <- sort(as.numeric(stats::na.omit(unique(c(A[,,i])))))[-1]
+        particleStats[loc,1:5] <- .Call('projectedPS',A[,,i],IDs,
+                                        PACKAGE='trackdem')
 
-       particleStats[loc,paste0('mu',c('R','G','B'))] <-
-                                                  as.matrix(extractMean(
-                                                  particleStats[loc,'patchID'],
-                                                  colorimages=colorimages[,,,i],
-                                                  images=A[,,i],ncolors=nc))
+        particleStats[loc,paste0('mu',c('R','G','B'))] <-
+                                                   as.matrix(extractMean(
+                                                   particleStats[loc,'patchID'],
+                                                   colorimages=colorimages[,,,i],
+                                                   images=A[,,i],ncolors=nc))
 
-       coords <- getCoords(m=A[,,i],d=dA)
-       ind <- A[,,i] > 0
-       particleStats[loc,'x'] <- tapply(coords[,2],A[,,i][ind],mean)
-       particleStats[loc,'y'] <- tapply(coords[,1],A[,,i][ind],mean)
-       particleStats[loc,'frame'] <- i
+        coords <- getCoords(m=A[,,i],d=dA)
+        ind <- A[,,i] > 0
+        particleStats[loc,'x'] <- tapply(coords[,2],A[,,i][ind],mean)
+        particleStats[loc,'y'] <- tapply(coords[,1],A[,,i][ind],mean)
+        particleStats[loc,'frame'] <- i
+      }
     }
 
     particleStats[,'perim.area.ratio'] <- particleStats[,'n.edges.perimeter'] /
@@ -869,3 +876,105 @@ shiny::runApp(
   }
  ))
 }
+
+
+##' Find pixel range
+##'
+##' This function can help to find the minimum and maximum particle size
+##' in pixels, to use in \code{identifyParticles}.
+##' @param colorimages Array containing original color images.
+##' @param frame Number specifying which frame to use. Default is frame 1.
+##' @author Marjolein Bruijning
+##' @examples
+##' \dontrun{
+##' dir <- "images"
+##' allFullImages <- loadImages (dirPictures=dir,nImages=1:30)
+##' stillBack <- createBackground(allFullImages)
+##' allImages <- subtractBackground(stillBack)
+##' findPixelRange(allFullImages,frame=10)
+##'	}
+##' @export
+findPixelRange <- function (colorimages,frame=1) {
+
+  invisible(runPixelRange(frame,colorimages))
+
+}
+
+runPixelRange <- function(frame,colorimages,ui,server) {
+shiny::runApp(
+  list(ui=shiny::fluidPage(
+  shiny::titlePanel(paste0("Find particle size range for particle detection")),
+  shiny::fluidRow(
+     shiny::column(3,
+     'The graph on the right shows the original image that can be used to zoom, by
+     drawing a polygon. Bottom graph shows the zoomed image. Draw a polygon
+     on this image and see its size in pixels. By drawing polygons over both the
+     smallest and largest particles of interest, an appropriate range can be found.
+     When finished, click on "Done".',
+      align='left',offset=0),
+      shiny::column(2,
+       shiny::actionButton("stop", "Done"),
+       shiny::br(),shiny::br()),
+      shiny::column(5,
+       shiny::h5('Choose region of interest to zoom (slower for larger images).'),
+       shiny::plotOutput("plot3",brush = shiny::brushOpts(id="plot3_brush",
+                        resetOnNew = TRUE,clip=TRUE)))
+      ),
+  shiny::hr(),
+  shiny::fluidRow(
+    shiny::column(6,
+    shiny::h3('Zoomed image'),
+    shiny::plotOutput("plot2",brush = shiny::brushOpts(id="plot2_brush",
+                     resetOnNew = TRUE,clip=TRUE))),
+    shiny::column(6,
+      shiny::verbatimTextOutput("info"))
+  )
+
+),
+  server=function(input, output) {
+
+   ranges <- shiny::reactiveValues(x=NULL,y=NULL)
+   coords <- shiny::reactiveValues(x=NULL,y=NULL)
+
+   output$plot3 <- shiny::renderPlot(suppressWarnings(graphics::plot(colorimages,frame=frame)))
+
+   # update plot range and coordinates
+   shiny::observe({
+     brush <- input$plot3_brush
+     if (!is.null(brush)) {
+       ranges$x <- floor(c(brush$xmin, brush$xmax) * ncol(colorimages))
+       ranges$y <- nrow(colorimages) - floor(c(brush$ymin, brush$ymax) *
+                   nrow(colorimages))
+
+     } else {
+       ranges$x <- c(1,ncol(colorimages))
+       ranges$y <- c(nrow(colorimages),1)
+     }
+
+     coords$x <- ranges$x / ncol(colorimages)
+     coords$y <- 1 - (ranges$y / nrow(colorimages))
+
+   })
+
+   output$plot2 <- shiny::renderPlot({
+      im <- colorimages[c(min(ranges$y):max(ranges$y)),
+                        c(min(ranges$x):max(ranges$x)),,frame,drop=FALSE]
+      im <- array(im,dim=c(dim(im)[1:3]))
+      im <- raster::brick(im)
+
+     suppressWarnings(raster::plotRGB(im,scale=1,asp=nrow(im)/ncol(im)))
+   })
+
+   output$info <- shiny::renderText({
+    x <- abs(diff(ranges$x)) * abs(input$plot2_brush$xmin-input$plot2_brush$xmax)
+    y <- abs(diff(ranges$y)) * abs(input$plot2_brush$ymin-input$plot2_brush$ymax)
+
+    paste0("Area: \n", round(x), ' by ',round(y),' = ',round(y*x),' pixels.')
+  })
+
+
+    shiny::observeEvent(input$stop, shiny::stopApp({
+     }))
+  }
+ ))
+}
